"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Condition = void 0;
const lodash_1 = require("lodash");
class Condition {
    constructor(search, attributeName) {
        this.search = search;
        this._not = false;
        this.key = attributeName;
        return this;
    }
    not() {
        this._not = !this._not;
        return this;
    }
    /**
     * Equal
     *
     * Works for all data types, including lists and maps.
     * Will look for 100% equality, when performing a search on a list, the docum
     *
     * - If the target attribute of the comparison is of type String, Number, or Binary, then the operator checks for
     *   a absolute match (similar to === in javascript).
     * - If the target attribute of the comparison is a set ("SS", "NS", or "BS"), then the operator evaluates to true if
     *   it finds every item specified and only the items specified (similar to _.isEqual in javascript).
     * - If the target attribute of the comparison is a Map, then the operator evaluates to true if the two objects map
     *   recursively (similar to _.isEqual in javascript).
     */
    eq(value) {
        if (this._not) {
            this.filter = ['<>', value];
        }
        else {
            this.filter = ['=', value];
        }
        return this.finalize();
    }
    /**
     * Less than
     *
     * Works for String, Number, or Binary (not a set type) attribute.
     * Does not work on sets.
     */
    lt(value) {
        if (this._not) {
            this.filter = ['>=', value];
        }
        else {
            this.filter = ['<', value];
        }
        return this.finalize();
    }
    /**
     * Less than or equal
     *
     * Works for String, Number, or Binary (not a set type) attribute.
     * Does not work on sets.
     */
    lte(value) {
        if (this._not) {
            this.filter = ['>', value];
        }
        else {
            this.filter = ['<=', value];
        }
        return this.finalize();
    }
    /**
     * Greater than
     *
     * Works for String, Number, or Binary (not a set type) attribute.
     * Does not work on sets.
     */
    gt(value) {
        if (this._not) {
            this.filter = ['<=', value];
        }
        else {
            this.filter = ['>', value];
        }
        return this.finalize();
    }
    /**
     * Greater than or equal
     *
     * Works for String, Number, or Binary (not a set type) attribute.
     * Does not work on sets.
     */
    gte(value) {
        if (this._not) {
            this.filter = ['<', value];
        }
        else {
            this.filter = ['>=', value];
        }
        return this.finalize();
    }
    /**
     * Checks for a prefix.
     *
     * Only works for String or Binary fields.
     * Does not work for numbers or sets.
     */
    beginsWith(value) {
        this.filter = ['beginsWith', value];
        return this.finalize();
    }
    /**
     * Checks for a subsequence, or value in a set.
     *
     * Condition value can contain only be a String, Number, or Binary (not a set type).
     *
     * - If the target attribute of the comparison is of type String, then the operator checks for a substring match.
     * - If the target attribute of the comparison is of type Binary, then the operator looks for a subsequence of the
     *   target that matches the input.
     * - If the target attribute of the comparison is a set ("SS", "NS", or "BS"), then the operator evaluates to true if
     *   it finds an exact match with any member of the set.
     *
     * When using `.not().contains(value)` this checks for the absence of a subsequence, or absence of a value in a set.
     */
    contains(value) {
        if (this._not) {
            this.filter = ['not contains', value];
        }
        else {
            this.filter = ['contains', value];
        }
        return this.finalize();
    }
    exists() {
        if (this._not) {
            this.filter = ['not exists'];
        }
        else {
            this.filter = ['exists'];
        }
        return this.finalize();
    }
    /**
     * Check if the attribute value matches of one of the given values.
     *
     * If the attribute value matches any of the values specified, the expression evaluates to true.
     *
     * Can provide up to 100 values to compare against.
     *
     * Works for String, Number, or Binary (not a set type) attributes. **Note: Does not work on sets.**
     */
    includes(...values) {
        if (this._not) {
            this.filter = ['excludes', (0, lodash_1.flatten)(values)];
        }
        else {
            this.filter = ['includes', (0, lodash_1.flatten)(values)];
        }
        return this.finalize();
    }
    /**
     * Check if the attribute value does NOT match any of the given values.
     *
     * This is a utility method, identical as if you did `.not().includes(…)`
     *
     * Works for String, Number, or Binary (not a set type) attributes. **Note: Does not work on sets.**
     */
    excludes(...values) {
        if (this._not) {
            this.filter = ['includes', (0, lodash_1.flatten)(values)];
        }
        else {
            this.filter = ['excludes', (0, lodash_1.flatten)(values)];
        }
        return this.finalize();
    }
    /**
     * Checks if a Set contains any of the provided values in a list.
     *
     * If any of the values specified are contained in the attribute's Set, the expression evaluates to true.
     *
     * Works for StringSet, NumberSet, or BinarySet attributes.
     */
    someOf(...values) {
        return this.intersects('some', values);
    }
    /**
     * Checks if a Set contains all of the provided values in a list.
     *
     * If every one of the values are contained with the attribute's Set, the expression evaluates to true.
     *
     * Works for StringSet, NumberSet, or BinarySet attributes.
     */
    allOf(...values) {
        return this.intersects('all', values);
    }
    /**
     * Greater than or equal to the first (lower) value, and less than or equal to the second (upper) value.
     *
     * Works for String, Number, or Binary (not a set type) attribute.
     * Does not work on sets.
     */
    between(lower, upper) {
        this.filter = ['between', lower, upper];
        return this.finalize();
    }
    /**
     * The attribute does not exist.
     *
     * NULL is supported for all data types, including lists and maps.
     */
    null() {
        if (this._not) {
            this.filter = ['not null'];
        }
        else {
            this.filter = ['null'];
        }
        return this.finalize();
    }
    intersects(allOrSome, values) {
        const key = this.key;
        const filters = []; // any is because of the Exclude on beginsWith
        const operator = this._not ? 'not contains' : 'contains';
        const options = (0, lodash_1.flatten)(values);
        for (let i = 0; i < options.length; i++) {
            const value = options[i];
            const filter = [operator, value];
            filters.push({ [key]: filter });
            if (allOrSome === 'some' && i !== options.length - 1) {
                filters.push('OR');
            }
        }
        if (allOrSome === 'some') {
            this.search.parenthesis(group => group.addFilterGroup(filters));
        }
        else {
            this.search.addFilterGroup(filters);
        }
        return this.search;
    }
    finalize() {
        const key = this.key;
        this.search.addFilterGroup([
            {
                [key]: this.filter,
            },
        ]);
        return this.search;
    }
}
exports.Condition = Condition;
//# sourceMappingURL=condition.js.map